@@ -39,28 +39,39 @@
                ,@body))
       (setf (gethash ,event *event-fn-table*) #',fn-name))))
 
-                                        ;(define-stump-event-handler :map-notify (event-window window override-redirect-p)
-                                        ;  )
+;;; Configure request
 
-(defun handle-mode-line-window (xwin x y width height)
-  (declare (ignore width))
-  (let ((ml (find-mode-line-window xwin)))
-    (when ml
-      (setf (xlib:drawable-height xwin) height)
-      (update-mode-line-position ml x y)
-      (resize-mode-line ml)
-      (sync-mode-line ml))))
+(flet ((has-x (mask) (= 1 (logand mask 1)))
+       (has-y (mask) (= 2 (logand mask 2)))
+       (has-w (mask) (= 4 (logand mask 4)))
+       (has-h (mask) (= 8 (logand mask 8)))
+       (has-bw (mask) (= 16 (logand mask 16)))
+       (has-stackmode (mask) (= 64 (logand mask 64))))
+  (defun configure-managed-window (win x y width height stack-mode value-mask)
+    ;; Grant the configure request but then maximize the window after the
+    ;; granting.
+    (when (or (has-w value-mask)
+              (has-h value-mask)
+              (has-stackmode value-mask))
+      ;; FIXME: I don't know why we need to clear the urgency bit
+      ;; here, but the old code would anytime a resize or raise
+      ;; request came in, so keep doing it. -sabetts
+      (when (window-urgent-p win)
+        (window-clear-urgency win)))
+    (when (or (has-x value-mask) (has-y value-mask))
+      (group-move-request (window-group win) win x y :parent))
+    (when (or (has-w value-mask) (has-h value-mask))
+      (group-resize-request (window-group win) win width height))
+    (when (has-stackmode value-mask)
+      (group-raise-request (window-group win) win stack-mode))
+    ;; Just to be on the safe side, hit the client with a fake
+    ;; configure event. The ICCCM says we have to do this at
+    ;; certain times; exactly when, I've sorta forgotten.
+    (update-configuration win))
 
-(defun handle-unmanaged-window (xwin x y width height border-width value-mask)
-  "Call this function for windows that stumpwm isn't
-  managing. Basically just give the window what it wants."
-  (labels ((has-x (mask) (= 1 (logand mask 1)))
-           (has-y (mask) (= 2 (logand mask 2)))
-           (has-w (mask) (= 4 (logand mask 4)))
-           (has-h (mask) (= 8 (logand mask 8)))
-           (has-bw (mask) (= 16 (logand mask 16)))
-           ;; (has-stackmode (mask) (= 64 (logand mask 64)))
-           )
+  (defun configure-unmanaged-window (xwin x y width height border-width value-mask)
+    "Call this function for windows that stumpwm isn't
+     managing. Basically just give the window what it wants."
     (xlib:with-state (xwin)
       (when (has-x value-mask)
         (setf (xlib:drawable-x xwin) x))
@@ -73,60 +84,27 @@
       (when (has-bw value-mask)
         (setf (xlib:drawable-border-width xwin) border-width)))))
 
-(defun update-configuration (win)
-  ;; Send a synthetic configure-notify event so that the window
-  ;; knows where it is onscreen.
-  (xwin-send-configuration-notify (window-xwin win)
-                                  (xlib:drawable-x (window-parent win))
-                                  (xlib:drawable-y (window-parent win))
-                                  (window-width win) (window-height win) 0))
-
 (define-stump-event-handler :configure-request (stack-mode #|parent|# window #|above-sibling|# x y width height border-width value-mask)
-  (labels ((has-x () (= 1 (logand value-mask 1)))
-           (has-y () (= 2 (logand value-mask 2)))
-           (has-w () (= 4 (logand value-mask 4)))
-           (has-h () (= 8 (logand value-mask 8)))
-           (has-stackmode () (= 64 (logand value-mask 64))))
-    ;; Grant the configure request but then maximize the window after the granting.
-    (dformat 3 "CONFIGURE REQUEST ~@{~S ~}~%" stack-mode window x y width height border-width value-mask)
-    (let ((win (find-window window)))
-      (cond
-        (win
-         (when (or (has-w) (has-h) (has-stackmode))
-           ;; FIXME: I don't know why we need to clear the urgency bit
-           ;; here, but the old code would anytime a resize or raise
-           ;; request came in, so keep doing it. -sabetts
-           (when (window-urgent-p win)
-             (window-clear-urgency win)))
-         (when (or (has-x) (has-y))
-           (group-move-request (window-group win) win x y :parent))
-         (when (or (has-w) (has-h))
-           (group-resize-request (window-group win) win width height))
-         (when (has-stackmode)
-           (group-raise-request (window-group win) win stack-mode))
-         ;; Just to be on the safe side, hit the client with a fake
-         ;; configure event. The ICCCM says we have to do this at
-         ;; certain times; exactly when, I've sorta forgotten.
-         (update-configuration win))
-        ((handle-mode-line-window win x y width height))
-        (t (handle-unmanaged-window window x y width height border-width value-mask))))))
+  (dformat 3 "CONFIGURE REQUEST ~@{~S ~}~%" stack-mode window x y width height border-width value-mask)
+  (let ((win (find-window window)))
+    (if win
+        (configure-managed-window win x y width height stack-mode value-mask)
+        (configure-unmanaged-window window x y width height border-width value-mask))))
 
 (define-stump-event-handler :configure-notify (stack-mode #|parent|# window #|above-sibling|# x y width height border-width value-mask)
   (dformat 4 "CONFIGURE NOTIFY ~@{~S ~}~%" stack-mode window x y width height border-width value-mask)
   (let ((screen (find-screen window)))
     (when screen
-      (let ((old-heads (copy-list (screen-heads screen))))
-        (setf (screen-heads screen) nil)
-        (let ((new-heads (make-screen-heads screen (screen-root screen))))
-          (setf (screen-heads screen) old-heads)
-          (cond
-            ((equalp old-heads new-heads)
-             (dformat 3 "Bogus configure-notify on root window of ~S~%" screen) t)
-            (t
-             (dformat 1 "Updating Xinerama configuration for ~S.~%" screen)
-             (if new-heads
-                 (head-force-refresh screen new-heads)
-                 (dformat 1 "Invalid configuration! ~S~%" new-heads)))))))))
+      (let ((old-heads (screen-heads screen))
+            (new-heads (make-screen-heads screen (screen-root screen))))
+        (cond
+          ((equalp old-heads new-heads)
+           (dformat 3 "Bogus configure-notify on root window of ~S~%" screen) t)
+          (t
+           (dformat 1 "Updating Xinerama configuration for ~S.~%" screen)
+           (if new-heads
+               (head-force-refresh screen new-heads)
+               (dformat 1 "Invalid configuration! ~S~%" new-heads))))))))
 
 (define-stump-event-handler :map-request (parent send-event-p window)
   (unless send-event-p
@@ -155,8 +133,9 @@
          ;; anyway.
          t)
         (t
-         (let ((window (process-mapped-window screen window)))
-           (group-raise-request (window-group window) window :map)))))))
+         (xlib:with-server-grabbed (*display*)
+           (let ((window (process-mapped-window screen window)))
+             (group-raise-request (window-group window) window :map))))))))
 
 (define-stump-event-handler :unmap-notify (send-event-p event-window window #|configure-p|#)
   ;; There are two kinds of unmap notify events: the straight up
@@ -176,12 +155,6 @@
               (decf (window-unmap-ignores window)))
             (withdraw-window window))))))
 
-;;(define-stump-event-handler :create-notify (#|window parent x y width height border-width|# override-redirect-p))
-;; (unless (or override-redirect-p
-;;          (internal-window-p (window-screen window) window))
-;;    (process-new-window (window-screen window) window))
-;;    (run-hook-with-args *new-window-hook* window)))
-
 (define-stump-event-handler :destroy-notify (send-event-p event-window window)
   (unless (or send-event-p
               (xlib:window-equal event-window window))
@@ -193,8 +166,8 @@
       (if win
           (destroy-window win)
           (progn
-            (let ((ml (find-mode-line-window window)))
-              (when ml (destroy-mode-line-window ml))))))))
+            (let ((ml (find-mode-line-by-window window)))
+              (when ml (destroy-mode-line ml))))))))
 
 (defun read-from-keymap (kmaps &optional update-fn)
   "Read a sequence of keys from the user, guided by the keymaps,
@@ -433,7 +406,7 @@ converted to an atom is removed."
          (if (plusp (screen-ignore-msg-expose screen))
              (decf (screen-ignore-msg-expose screen))
              (redraw-current-message screen)))
-        ((setf ml (find-mode-line-window window))
+        ((setf ml (find-mode-line-by-window window))
          (setf screen (mode-line-screen ml))
          (redraw-mode-line ml t)))
       ;; Show the area.
@@ -565,9 +538,9 @@ converted to an atom is removed."
 (define-stump-event-handler :focus-out (window mode kind)
   (dformat 5 "~@{~s ~}~%" window mode kind))
 
-(define-stump-event-handler :focus-in (window mode)
+(define-stump-event-handler :focus-in (window mode kind)
   (let ((win (find-window window)))
-    (when (and win (eq mode :normal))
+    (when (and win (eq mode :normal) (not (eq kind :pointer)))
       (let ((screen (window-screen win)))
         (unless (eq win (screen-focus screen))
           (setf (screen-focus screen) win))))))
@@ -597,18 +570,13 @@ the window in it's frame."
       ((and (setf screen (find-screen window)) (not child))
        (group-button-press (screen-current-group screen) x y :root)
        (run-hook-with-args *root-click-hook* screen code x y))
-      ((setf ml (find-mode-line-window window))
+      ((setf ml (find-mode-line-by-window window))
        (run-hook-with-args *mode-line-click-hook* ml code x y))
       ((setf win (find-window-by-parent window (top-windows)))
        (group-button-press (window-group win) x y win))))
   ;; Pass click to client
   (xlib:allow-events *display* :replay-pointer time))
 
-;; Handling event :KEY-PRESS
-;; (:DISPLAY #<XLIB:DISPLAY :0 (The X.Org Foundation R60700000)> :EVENT-KEY :KEY-PRESS :EVENT-CODE 2 :SEND-EVENT-P NIL :CODE 45 :SEQUENCE 1419 :TIME 98761213 :ROOT #<XLIB:WINDOW :0 96> :WINDOW #<XLIB:WINDOW :0 6291484> :EVENT-WINDOW #<XLIB:WINDOW :0 6291484> :CHILD
-;;  #<XLIB:WINDOW :0 6291485> :ROOT-X 754 :ROOT-Y 223 :X 753 :Y 222 :STATE 4 :SAME-SCREEN-P T)
-;; H
-
 (defun handle-event (&rest event-slots &key display event-key &allow-other-keys)
   (declare (ignore display))
   (dformat 1 ">>> ~S~%" event-key)
