@@ -1,6 +1,9 @@
 ;;; implementation of a floating style window management group
+(defpackage #:stumpwm.floating-group
+  (:use :cl :stumpwm)
+  (:export #:float-group))
 
-(in-package :stumpwm)
+(in-package :stumpwm.floating-group)
 
 ;;; floating window
 
@@ -8,33 +11,30 @@
   ((last-width :initform 0 :accessor float-window-last-width)
    (last-height :initform 0 :accessor float-window-last-height)
    (last-x :initform 0 :accessor float-window-last-x)
-   (last-y :initform 0 :accessor float-window-last-y))
-  )
+   (last-y :initform 0 :accessor float-window-last-y)))
 
 (defvar *float-window-border* 1)
 (defvar *float-window-title-height* 10)
 
 ;; some book keeping functions
 (defmethod (setf window-x) :before (val (window float-window))
-  (unless (eql (window-x window) val)
-    (setf (float-window-last-x window) (window-x window))))
+  (setf (float-window-last-x window) (window-x window)))
 
 (defmethod (setf window-y) :before (val (window float-window))
-  (unless (eql (window-y window) val)
-    (setf (float-window-last-y window) (window-y window))))
+  (setf (float-window-last-y window) (window-y window)))
 
 (defmethod (setf window-width) :before (val (window float-window))
-  (unless (eql (window-width window) val)
-    (setf (float-window-last-width window) (window-width window))))
+  (setf (float-window-last-width window) (window-width window)))
 
 (defmethod (setf window-height) :before (val (window float-window))
-  (unless (eql (window-height window) val)
-    (setf (float-window-last-height window) (window-height window))))
+  (setf (float-window-last-height window) (window-height window)))
 
 (defun float-window-move-resize (win &key x y width height (border *float-window-border*))
   ;; x and y position the parent window while width, height resize the
   ;; xwin (meaning the parent will have a larger width).
-  (with-slots (xwin parent) win
+  (with-accessors ((xwin window-xwin)
+                   (parent window-parent))
+      win
     (xlib:with-state (parent)
       (xlib:with-state (xwin)
         (when x
@@ -61,28 +61,46 @@
     (xlib:clear-area (window-parent window))))
 
 (defmethod window-sync ((window float-window) hint)
-  (declare (ignore hint))
-  )
+  (declare (ignore hint)))
 
 (defmethod window-head ((window float-window))
-  (dolist (head (screen-heads (group-screen (window-group window))))
-    (when (and
-           (>= (window-x window) (frame-x head))
-           (>= (window-y window) (frame-y head))
-           (<= (+ (window-x window) (window-width window))
-               (+ (frame-x head) (frame-width head)))
-           (<= (+ (window-y window) (window-height window))
-               (+ (frame-y head) (frame-height head))))
-      (return head))))
+  (let ((left (window-x window))
+        (right (+ (window-x window) (window-width window)))
+        (top (window-y window))
+        (bottom (+ (window-y window) (window-height window)))
+        (heads (screen-heads (group-screen (window-group window)))))
+    (flet ((within-frame-p (y x head)
+             (and (>= x (frame-x head))
+                  (< x (+ (frame-x head) (frame-width head)))
+                  (>= y (frame-y head))
+                  (< y (+ (frame-y head) (frame-height head))))))
+      (or (find-if (lambda (head)
+                     (or (within-frame-p top left head)
+                         (within-frame-p top right head)
+                         (within-frame-p bottom left head)
+                         (within-frame-p bottom right head)))
+                   heads)
+          ;; Didn't find any head, so give up and return the first one
+          ;; in the list.
+          (first heads)))))
 
 (defmethod window-visible-p ((win float-window))
   (eql (window-state win) +normal-state+))
 
 (defmethod (setf window-fullscreen) :after (val (window float-window))
-  (with-slots (last-x last-y last-width last-height parent) window
+  (with-accessors ((last-x float-window-last-x)
+                   (last-y float-window-last-y)
+                   (last-width float-window-last-width)
+                   (last-height float-window-last-height)
+                   (parent window-parent))
+      window
     (if val
         (let ((head (window-head window)))
-          (with-slots (x y width height) window
+          (with-accessors ((x window-x)
+                           (y window-y)
+                           (width window-width)
+                           (height window-height))
+              window
             (format t "major on: ~a ~a ~a ~a~%" x y width height))
           (set-window-geometry window :x 0 :y 0)
           (float-window-move-resize window
@@ -102,14 +120,15 @@
                                     :width last-width
                                     :height last-height)))))
 
+(defmethod really-raise-window ((window float-window))
+  (raise-window window))
+
 ;;; floating group
 
 (defclass float-group (group)
-  ((current-window :accessor float-group-current-window))
-  )
+  ((current-window :accessor float-group-current-window)))
 
-(defmethod group-startup ((group float-group))
-  )
+(defmethod group-startup ((group float-group)))
 
 (defmethod group-add-window ((group float-group) window &key &allow-other-keys)
   (change-class window 'float-window)
@@ -128,8 +147,7 @@
 (defmethod group-wake-up ((group float-group))
   (&float-focus-next group))
 
-(defmethod group-suspend ((group float-group))
-  )
+(defmethod group-suspend ((group float-group)))
 
 (defmethod group-current-window ((group float-group))
   (screen-focus (group-screen group)))
@@ -140,7 +158,11 @@
       (first (screen-heads (group-screen group)))))
 
 (defun float-window-align (window)
-  (with-slots (parent xwin width height) window
+  (with-accessors ((parent window-parent)
+                   (xwin window-xwin)
+                   (width window-width)
+                   (height window-height))
+      window
     (set-window-geometry window :x *float-window-border* :y *float-window-title-height*)
     (xlib:with-state (parent)
       (setf (xlib:drawable-width parent) (+ width (* 2 *float-window-border*))
@@ -167,7 +189,7 @@
   )
 
 (defmethod group-focus-window ((group float-group) window)
-  (focus-window window))
+  (focus-window window nil))
 
 (defmethod group-root-exposure ((group float-group))
   )
@@ -192,7 +214,7 @@
   (let ((screen (group-screen group))
         (initial-width (xlib:drawable-width (window-parent window)))
         (initial-height (xlib:drawable-height (window-parent window))))
-    (when (eq *mouse-focus-policy* :click)
+    (when (member *mouse-focus-policy* '(:click :sloppy))
       (focus-window window))
 
     ;; When in border
@@ -219,7 +241,8 @@
                      (case event-key
                        (:button-release :done)
                        (:motion-notify
-                        (with-slots (parent) window
+                        (with-accessors ((parent window-parent))
+                            window
                           (xlib:with-state (parent)
                             ;; Either move or resize the window
                             (cond
@@ -258,13 +281,20 @@
                   (window-y window) (xlib:drawable-y (window-parent window)))))))))
 
 (defmethod group-button-press ((group float-group) x y where)
-  (declare (ignore x y where))
-  )
+  (declare (ignore x y where)))
 
+;;; Bindings
+
+(pushnew '(float-group *float-group-top-map*) *group-top-maps*)
+(defvar *float-group-top-map* (make-sparse-keymap))
+(defvar *float-group-root-map* (make-sparse-keymap))
+
+
+(in-package :stumpwm)
 (defcommand gnew-float (name) ((:rest "Group Name: "))
   "Create a floating window group with the specified name and switch to it."
-  (add-group (current-screen) name :type 'float-group))
+  (add-group (current-screen) name :type 'stumpwm.floating-group:float-group))
 
 (defcommand gnewbg-float (name) ((:rest "Group Name: "))
   "Create a floating window group with the specified name, but do not switch to it."
-  (add-group (current-screen) name :background t :type 'float-group))
+  (add-group (current-screen) name :background t :type 'stumpwm.floating-group:float-group))
