@@ -54,37 +54,46 @@
           m)))
 
 (defstruct menu-state
-  table prompt selected view-start view-end
+  unfiltered-table table filter-pred prompt selected view-start view-end
   (current-input (make-array 10 :element-type 'character :adjustable t :fill-pointer 0)))
 
+(defun menu-scrolling-required (menu)
+  (and *menu-maximum-height*
+       (> (length (menu-state-table menu))
+          *menu-maximum-height*)))
+
+(defun menu-height (menu)
+  (let ((len (length (menu-state-table menu))))
+    (min (or *menu-maximum-height* len) len)))
+
+(defun menu-prompt-visible (menu)
+  (or (menu-state-prompt menu)
+      (> (length (menu-state-current-input menu)) 0)))
+
 (defun bound-check-menu (menu)
   "Adjust the menu view and selected item based
 on current view and new selection."
   (let ((len (length (menu-state-table menu))))
+    ;; Wrap around
     (setf (menu-state-selected menu)
           (cond ((< (menu-state-selected menu) 0) (1- len))
                 ((>= (menu-state-selected menu) len) 0)
                 (t (menu-state-selected menu))))
-    (when (and *menu-maximum-height*
-               (> len *menu-maximum-height*))  ; scrolling required
-      (let ((sel (menu-state-selected menu)))
-        (setf (values (menu-state-view-start menu)
-                      (menu-state-view-end menu))
-              (cond ((< sel *menu-maximum-height*)
-                     (values 0 *menu-maximum-height*))
-                    ((> sel (- len *menu-maximum-height*))
-                     (values (- len *menu-maximum-height*) len))
-                    ((< sel (menu-state-view-start menu))
-                     (values (- sel *menu-scrolling-step*)
-                             (- (+ sel *menu-maximum-height*)
-                                *menu-scrolling-step*)))
-                    ((>= sel (menu-state-view-end menu))
-                     (values (+ (- sel *menu-maximum-height*)
-                                *menu-scrolling-step*)
-                             (+ sel *menu-scrolling-step*)))
-                    (t
-                     (values (menu-state-view-start menu)
-                             (menu-state-view-end menu)))))))))
+    (setf (values (menu-state-view-start menu)
+                  (menu-state-view-end menu))
+          (if (zerop len)
+              (values 0 0)
+              (let* ((menu-height (menu-height menu))
+                     (sel (menu-state-selected menu))
+                     (start (- sel 1))
+                     (end (+ sel menu-height -1)))
+                (multiple-value-bind
+                      (start end) (cond ((< start 0) (values 0 menu-height))
+                                        ((> end len) (values (- len menu-height) len))
+                                        (t (values start end)))
+                  (assert (<= 0 start (- len menu-height)) (start))
+                  (assert (<= menu-height end len) (end))
+                  (values start end)))))))
 
 (defun menu-up (menu)
   (setf (fill-pointer (menu-state-current-input menu)) 0)
@@ -149,90 +158,94 @@ backspace or F9), return it otherwise return nil"
 
 (defun check-menu-complete (menu key-seq)
   "If the user entered a key not mapped in @var{*menu-map}, check if
-  he's trying to type an entry's name. Match is case insensitive as
-  long as the user types lower-case characters. If @var{key-seq} is
-  nil, some other function has manipulated the current-input and is
-  requesting a re-computation of the match."
+  he's trying to type an entry's name. Match is case insensitive. If
+  @var{key-seq} is nil, some other function has manipulated the
+  current-input and is requesting a re-computation of the match."
   (let ((input-char (and key-seq (get-input-char key-seq))))
     (when input-char
       (vector-push-extend input-char (menu-state-current-input menu)))
-    (when (or input-char (not key-seq))
-      (do* ((cur-pos 0 (1+ cur-pos))
-	    (rest-elem (menu-state-table menu)
-		       (cdr rest-elem))
-	    (cur-elem (car rest-elem) (car rest-elem))
-	    (cur-elem-name (menu-element-name cur-elem) (menu-element-name cur-elem))
-	    (current-input-length (length (menu-state-current-input menu)))
-	    (match-regex (ppcre:create-scanner (menu-state-current-input menu)
-					       :case-insensitive-mode
-					       (string= (string-downcase (menu-state-current-input menu))
-							(menu-state-current-input menu)))))
-	   ((not cur-elem))
-	(when (and (>= (length cur-elem-name) current-input-length)
-		   (ppcre:scan match-regex cur-elem-name))
-	  (setf (menu-state-selected menu) cur-pos)
-          (bound-check-menu menu)
-	  (return))))))
+    (handler-case
+        (when (or input-char (not key-seq))
+          (labels ((match-p (table-item)
+                     (funcall (menu-state-filter-pred menu)
+                              (car table-item)
+                              (second table-item)
+                              (menu-state-current-input menu))))
+            (setf (menu-state-table menu) (remove-if-not #'match-p (menu-state-unfiltered-table menu))
+                  (menu-state-selected menu) 0)
+            (bound-check-menu menu)))
+      (cl-ppcre:ppcre-syntax-error ()))))
 
-(defun select-from-menu (screen table &optional prompt
+(defun menu-item-matches-regexp (item-string item-object user-input)
+  "The default filter predicate for SELECT-FROM-MENU. When using this
+predicate, an item is visible when it matches all of the regular
+expressions in USER-INPUT (multiple regexps are separated by one or
+more spaces; ARGUMENT-POP is used to split the string)."
+  (match-all-regexps user-input item-string))
+
+(defun select-from-menu (screen table &optional (prompt "Search:")
                                         (initial-selection 0)
-                                        extra-keymap)
+                                        extra-keymap
+                                        (filter-pred #'menu-item-matches-regexp))
   "Prompt the user to select from a menu on SCREEN. TABLE can be
 a list of values or an alist. If it's an alist, the CAR of each
 element is displayed in the menu. What is displayed as menu items
 must be strings.
 EXTRA-KEYMAP can be a keymap whose bindings will take precedence
 over the default bindings.
+FILTER-PRED should be a a function returning T when a certain menu
+item should be visible to the user.  It should accept arguments
+ITEM-STRING (the string shown to the user), ITEM-OBJECT (the object
+corresponding to the menu item), and USER-INPUT (the current user
+input). The default is MENU-ITEM-MATCHES-REGEXP.
 Returns the selected element in TABLE or nil if aborted. "
   (check-type screen screen)
   (check-type table list)
   (check-type prompt (or null string))
   (check-type initial-selection integer)
-  (let* ((menu-options (mapcar #'menu-element-name table))
-         (menu-require-scrolling (and *menu-maximum-height*
-                                       (> (length menu-options)
-                                          *menu-maximum-height*)))
-         (menu (make-menu-state
-                :table table
-                :prompt prompt
-                :view-start (if menu-require-scrolling initial-selection 0)
-                :view-end (if menu-require-scrolling
-                              (+ initial-selection *menu-maximum-height*)
-                              (length menu-options))
-                :selected initial-selection))
-         (*record-last-msg-override* t)
-         (*suppress-echo-timeout* t))
-    (bound-check-menu menu)
-    (catch :menu-quit
-      (unwind-protect
-           (with-focus (screen-key-window screen)
-             (loop
-                (let ((strings (subseq menu-options
-                                       (menu-state-view-start menu)
-                                       (menu-state-view-end menu)))
-                      (highlight (- (menu-state-selected menu)
-                                    (menu-state-view-start menu))))
-                  (unless (= 0 (menu-state-view-start menu))
-                    (setf strings (cons "..." strings))
-                    (incf highlight))
-                  (unless (= (length menu-options) (menu-state-view-end menu))
-                    (setf strings (nconc strings '("..."))))
-                  (unless (= (fill-pointer (menu-state-current-input menu)) 0)
-                    (setf strings
-                          (cons (format nil "Search: ~a"
-                                        (menu-state-current-input menu))
-                                strings))
-                    (incf highlight))
-                  (when prompt
-                    (setf strings (cons prompt strings))
-                    (incf highlight))
-                  (echo-string-list screen strings highlight))
-                (multiple-value-bind (action key-seq) (read-from-keymap
-                                                       (if extra-keymap
-                                                           (list extra-keymap
-                                                                 *menu-map*)
-                                                           (list *menu-map*)))
-                  (if action
-                      (funcall action menu)
-                      (check-menu-complete menu (first key-seq))))))
-        (unmap-all-message-windows)))))
+
+  (when table
+    (let* ((*record-last-msg-override* t)
+           (*suppress-echo-timeout* t)
+           (menu (make-menu-state
+                  :unfiltered-table table
+                  :table table
+                  :filter-pred filter-pred
+                  :prompt prompt
+                  :view-start 0
+                  :view-end 0
+                  :selected initial-selection))
+           (keymap (if extra-keymap
+                       (list extra-keymap *menu-map*)
+                       (list *menu-map*))))
+      (bound-check-menu menu)
+      (catch :menu-quit
+        (unwind-protect
+             (with-focus (screen-key-window screen)
+               (loop
+                  (let* ((sel (menu-state-selected menu))
+                         (start (menu-state-view-start menu))
+                         (end (menu-state-view-end menu))
+                         (len (length (menu-state-table menu)))
+                         (prompt-line (when (menu-prompt-visible menu)
+                                        (format nil "~@[~A ~]~A"
+                                                prompt (menu-state-current-input menu))))
+                         (strings (mapcar #'menu-element-name
+                                          (subseq (menu-state-table menu)
+                                                  start end)))
+                         (highlight (- sel start)))
+                    (unless (zerop start)
+                      (setf strings (cons "..." strings))
+                      (incf highlight))
+                    (unless (= len end)
+                      (setf strings (nconc strings '("..."))))
+                    (when prompt-line
+                      (push prompt-line strings)
+                      (incf highlight))
+                    (echo-string-list screen strings highlight))
+                  (multiple-value-bind (action key-seq) (read-from-keymap keymap)
+                    (if action
+                        (progn (funcall action menu)
+                               (bound-check-menu menu))
+                        (check-menu-complete menu (first key-seq))))))
+          (unmap-all-message-windows))))))
