@@ -19,7 +19,9 @@
 
 (defpackage :stumpwm
   (:use :cl)
-  (:shadow #:yes-or-no-p #:y-or-n-p))
+  (:shadow #:yes-or-no-p #:y-or-n-p)
+  (:export
+   #:call-in-main-thread))
 
 (defpackage :stumpwm-user
   (:use :cl :stumpwm))
