@@ -25,7 +25,7 @@
 
 ;; Web jump (works for Google and Imdb)
 (defmacro make-web-jump (name prefix)
-  `(defcommand ,name (search) ((:rest ,(concatenate 'string name " search: ")))
+  `(defcommand ,(intern name) (search) ((:rest ,(concatenate 'string name " search: ")))
     (substitute #\+ #\Space search)
     (run-shell-command (concatenate 'string ,prefix search))))
 
@@ -72,4 +72,4 @@
 
 (define-frame-preference "Emacs"
   (1 t t :restore "emacs-editing-dump" :title "...xdvi")
-  (0 t t :create "emacs-dump" :class "Emacs"))
\ No newline at end of file
+  (0 t t :create "emacs-dump" :class "Emacs"))
