@@ -123,7 +123,8 @@
     (let* ((frame (find-frame group x y)))
       (when frame
         (focus-frame group frame)
-        (update-all-mode-lines)))))
+        (unless (eq *mouse-focus-policy* :click)
+          (update-all-mode-lines))))))
 
 (defmethod group-button-press ((group tile-group) x y (where window))
   (declare (ignore x y))
@@ -173,7 +174,11 @@
     (sync-frame-windows group f)))
 
 ;;;;;
-
+;; (defun tile-group-frame-head (group head)
+;;   (let ((index (position head (group-heads group)))
+;;         (frame-tree (tile-group-frame-tree group)))
+;;     (when (> index (length frame-tree))
+;;       (elt frame-tree index))))
 (defun tile-group-frame-head (group head)
   (elt (tile-group-frame-tree group) (position head (group-heads group))))
 
