@@ -299,7 +299,8 @@ instance. @var{all-groups} overrides this default. Similarily for
                     (second other-matches)
                     (first matches))))
       (if win
-          (if (eq (type-of (window-group win)) 'float-group)
+          (if (eq (type-of (window-group win))
+                  'stumpwm.floating-group:float-group)
               (focus-all win)
               (goto-win win))
           (run-shell-command cmd)))))
@@ -346,11 +347,11 @@ submitting the bug report."
 
 (defmacro defprogram-shortcut (name &key (command (string-downcase (string name)))
                                          (props `'(:class ,(string-capitalize command)))
-                                         (map *top-map*)
-                                         (key (kbd (concat "H-" (subseq command 0 1))))
+                                         (map '*top-map*)
+                                         (key `(kbd ,(concat "H-" (subseq command 0 1))))
                                          (pullp nil)
                                          (pull-name (intern1 (concat (string name) "-PULL")))
-                                         (pull-key (kbd (concat "H-M-" (subseq command 0 1)))))
+                                         (pull-key `(kbd ,(concat "H-M-" (subseq command 0 1)))))
   "Define a command and key binding to run or raise a program. If
 @var{pullp} is set, also define a command and key binding to run or
 pull the program."
@@ -358,22 +359,25 @@ pull the program."
      (defcommand ,name () ()
        (run-or-raise ,command ,props))
      (define-key ,map ,key ,(string-downcase (string name)))
-     (when ,pullp
-       (defcommand (,pull-name tile-group) () ()
-          (run-or-pull ,command ,props))
-       (define-key ,map ,pull-key ,(string-downcase (string pull-name))))))
+     ,(when pullp
+        `(progn
+           (defcommand (,pull-name tile-group) () ()
+             (run-or-pull ,command ,props))
+           (define-key ,map ,pull-key ,(string-downcase (string pull-name)))))))
 
 (defcommand show-window-properties () ()
   "Shows the properties of the current window. These properties can be
 used for matching windows with run-or-raise or window placement
 rules."
   (let ((w (current-window)))
-    (message-no-timeout "class: ~A~%instance: ~A~%type: :~A~%role: ~A~%title: ~A"
-                        (window-class w)
-                        (window-res w)
-                        (string (window-type w))
-                        (window-role w)
-                        (window-title w))))
+    (if (not w)
+        (message "No active window!")
+        (message-no-timeout "class: ~A~%instance: ~A~%type: :~A~%role: ~A~%title: ~A"
+                            (window-class w)
+                            (window-res w)
+                            (string (window-type w))
+                            (window-role w)
+                            (window-title w)))))
 
 (defcommand list-window-properties () ()
   "List all the properties of the current window and their values,
